/*
 * libnqs/comms/analyzetcm.c
 *
 * DESCRIPTION:
 *
 *	Completely analyze and diagnose a transaction completion
 *	code.
 *
 *	Original Author:
 *	-------
 *	Brent A. Kingsbury, Sterling Software Incorporated.
 *	April 28, 1986.
 */

#include <libnqs/license.h>
#include <libnqs/proto.h>			/* Everyone gets this */
#include <libnqs/informcc.h>		/* NQS information completion codes */
#include <libnqs/transactcc.h>		/* NQS transaction completion codes */

static void exceedpreamble ( FILE *stream, char *prefix );
static void tcm_boundpreamble ( FILE *stream, char *prefix );

/*** analyzetcm
 *
 *
 *	void analyzetcm():
 *
 *	Completely analyze and diagnose a transaction completion
 *	code.
 */
void analyzetcm (
	long code,		/* Completion code */
	FILE *stream,		/* Output stream */
	char *prefix)		/* Output prefix */
{

	register short reason;		/* Reason bits of completion code */

  	/* debugging checks added in 3.41.0 */
  	assert(stream != NULL);
  	assert(prefix != NULL);
  
	reason = (code & XCI_REASON_MASK);
	if (reason < 0 || reason > (TCM_MAXTCM & XCI_REASON_MASK) ||
	   (code & XCI_TRANSA_MASK) == 0) {
		/*
		 *  The transaction code is invalid.
		 */
		reason = (TCML_UNDEFINED & XCI_REASON_MASK);
		code = TCML_UNDEFINED;
	}
	else if (reason == (TCML_UNDEFINED & XCI_REASON_MASK)) {
		/*
		 *  Clear information bits if undefined transaction code.
		 */
		code &= ~XCI_INFORM_MASK;
	}
	/*
	 *  Analyze and describe a specific transaction code.
	 */
	tcmmsgs (code, stream, prefix);
	/*
	 *  Analyze and describe any additional information bits
	 *  set in the transaction completion code.
	 */
	if (reason == (TCML_QUOTALIMIT & XCI_REASON_MASK)) {
		tcimsgs (code, stream, prefix, exceedpreamble);
	}
	else if (reason == (TCML_SUBMITTED & XCI_REASON_MASK)) {
		tcimsgs (code, stream, prefix, tcm_boundpreamble);
	}
}


/*** tcm_boundpreamble
 *
 *
 *	void tcm_boundpreamble():
 *	Display bounded limit preamble.
 */
static void tcm_boundpreamble (FILE *stream, char *prefix)
{
  	assert (stream != NULL);
  
	fputc ('\n', stream);
	fputs (prefix, stream);
	fputs ("In order to successfully queue or deliver the request,\n",
		stream);
	fputs (prefix, stream);
	fputs ("it was necessary to alter the following explicit request\n",
		stream);
	fputs (prefix, stream);
	fputs ("quota limit(s), binding them to values within the ranges\n",
		stream);
	fputs (prefix, stream);
	fputs ("enforceable at the execution machine:\n\n", stream);
	fflush (stream);		/* For output to logfile */
}


/*** exceedpreamble
 *
 *
 *	void exceedpreamble():
 *	Display limit too large to queue preamble.
 */
static void exceedpreamble (FILE *stream, char *prefix)
{
  	assert(stream != NULL);
  	assert(prefix != NULL);
  
	fputc ('\n', stream);
	fputs (prefix, stream);
	fputs ("The request could not be queued or delivered because the\n",
		stream);
	fputs (prefix, stream);
	fputs ("following explicit request quota limit(s) exceeded the\n",
		stream);
	fputs (prefix, stream);
	fputs ("corresponding limits of the target queue:\n\n", stream);
	fflush (stream);		/* For output to logfile */
}
